@@ -31,62 +31,53 @@ static const uint32_t BITMAPINFOSIZE = bmp::InfoHeaderLength::WIN_ICO;
 // ----------------------------------------
 
 // Obtains the number of colors from the bits per pixel
-uint16_t
-nsICODecoder::GetNumColors()
-{
+uint16_t nsICODecoder::GetNumColors() {
   uint16_t numColors = 0;
   if (mBPP <= 8) {
     switch (mBPP) {
-    case 1:
-      numColors = 2;
-      break;
-    case 4:
-      numColors = 16;
-      break;
-    case 8:
-      numColors = 256;
-      break;
-    default:
-      numColors = (uint16_t)-1;
+      case 1:
+        numColors = 2;
+        break;
+      case 4:
+        numColors = 16;
+        break;
+      case 8:
+        numColors = 256;
+        break;
+      default:
+        numColors = (uint16_t)-1;
     }
   }
   return numColors;
 }
 
 nsICODecoder::nsICODecoder(RasterImage* aImage)
-  : Decoder(aImage)
-  , mLexer(Transition::To(ICOState::HEADER, ICOHEADERSIZE),
-           Transition::TerminateSuccess())
-  , mDirEntry(nullptr)
-  , mNumIcons(0)
-  , mCurrIcon(0)
-  , mBPP(0)
-  , mMaskRowSize(0)
-  , mCurrMaskLine(0)
-  , mIsCursor(false)
-  , mHasMaskAlpha(false)
-{ }
+    : Decoder(aImage),
+      mLexer(Transition::To(ICOState::HEADER, ICOHEADERSIZE),
+             Transition::TerminateSuccess()),
+      mDirEntry(nullptr),
+      mNumIcons(0),
+      mCurrIcon(0),
+      mBPP(0),
+      mMaskRowSize(0),
+      mCurrMaskLine(0),
+      mIsCursor(false),
+      mHasMaskAlpha(false) {}
 
-nsresult
-nsICODecoder::FinishInternal()
-{
+nsresult nsICODecoder::FinishInternal() {
   // We shouldn't be called in error cases
   MOZ_ASSERT(!HasError(), "Shouldn't call FinishInternal after error!");
 
   return GetFinalStateFromContainedDecoder();
 }
 
-nsresult
-nsICODecoder::FinishWithErrorInternal()
-{
+nsresult nsICODecoder::FinishWithErrorInternal() {
   // No need to assert !mInFrame here because this condition is enforced by
   // mContainedDecoder.
   return GetFinalStateFromContainedDecoder();
 }
 
-nsresult
-nsICODecoder::GetFinalStateFromContainedDecoder()
-{
+nsresult nsICODecoder::GetFinalStateFromContainedDecoder() {
   if (!mContainedDecoder) {
     return NS_OK;
   }
@@ -108,17 +99,14 @@ nsICODecoder::GetFinalStateFromContainedDecoder()
   }
 
   // Propagate errors.
-  nsresult rv = HasError() || mContainedDecoder->HasError()
-              ? NS_ERROR_FAILURE
-              : NS_OK;
+  nsresult rv =
+      HasError() || mContainedDecoder->HasError() ? NS_ERROR_FAILURE : NS_OK;
 
   MOZ_ASSERT(NS_FAILED(rv) || !mCurrentFrame || mCurrentFrame->IsFinished());
   return rv;
 }
 
-LexerTransition<ICOState>
-nsICODecoder::ReadHeader(const char* aData)
-{
+LexerTransition<ICOState> nsICODecoder::ReadHeader(const char* aData) {
   // If the third byte is 1, this is an icon. If 2, a cursor.
   if ((aData[2] != 1) && (aData[2] != 2)) {
     return Transition::TerminateFailure();
@@ -128,7 +116,7 @@ nsICODecoder::ReadHeader(const char* aData)
   // The fifth and sixth bytes specify the number of resources in the file.
   mNumIcons = LittleEndian::readUint16(aData + 4);
   if (mNumIcons == 0) {
-    return Transition::TerminateSuccess(); // Nothing to do.
+    return Transition::TerminateSuccess();  // Nothing to do.
   }
 
   // Downscale-during-decode can end up decoding different resources in the ICO
@@ -142,9 +130,7 @@ nsICODecoder::ReadHeader(const char* aData)
   return Transition::To(ICOState::DIR_ENTRY, ICODIRENTRYSIZE);
 }
 
-size_t
-nsICODecoder::FirstResourceOffset() const
-{
+size_t nsICODecoder::FirstResourceOffset() const {
   MOZ_ASSERT(mNumIcons > 0,
              "Calling FirstResourceOffset before processing header");
 
@@ -153,9 +139,7 @@ nsICODecoder::FirstResourceOffset() const
   return ICOHEADERSIZE + mNumIcons * ICODIRENTRYSIZE;
 }
 
-LexerTransition<ICOState>
-nsICODecoder::ReadDirEntry(const char* aData)
-{
+LexerTransition<ICOState> nsICODecoder::ReadDirEntry(const char* aData) {
   mCurrIcon++;
 
   // Ensure the resource has an offset past the ICO headers.
@@ -163,15 +147,15 @@ nsICODecoder::ReadDirEntry(const char* aData)
   if (offset >= FirstResourceOffset()) {
     // Read the directory entry.
     IconDirEntryEx e;
-    e.mWidth       = aData[0];
-    e.mHeight      = aData[1];
-    e.mColorCount  = aData[2];
-    e.mReserved    = aData[3];
-    e.mPlanes      = LittleEndian::readUint16(aData + 4);
-    e.mBitCount    = LittleEndian::readUint16(aData + 6);
-    e.mBytesInRes  = LittleEndian::readUint32(aData + 8);
+    e.mWidth = aData[0];
+    e.mHeight = aData[1];
+    e.mColorCount = aData[2];
+    e.mReserved = aData[3];
+    e.mPlanes = LittleEndian::readUint16(aData + 4);
+    e.mBitCount = LittleEndian::readUint16(aData + 6);
+    e.mBytesInRes = LittleEndian::readUint32(aData + 8);
     e.mImageOffset = offset;
-    e.mSize        = IntSize(e.mWidth, e.mHeight);
+    e.mSize = IntSize(e.mWidth, e.mHeight);
 
     // Only accept entries with sufficient resource data to actually contain
     // some image data.
@@ -194,16 +178,14 @@ nsICODecoder::ReadDirEntry(const char* aData)
   return Transition::To(ICOState::DIR_ENTRY, ICODIRENTRYSIZE);
 }
 
-LexerTransition<ICOState>
-nsICODecoder::IterateUnsizedDirEntry()
-{
+LexerTransition<ICOState> nsICODecoder::IterateUnsizedDirEntry() {
   MOZ_ASSERT(!mUnsizedDirEntries.IsEmpty());
 
   if (!mDirEntry) {
     // The first time we are here, there is no entry selected. We must prepare a
     // new iterator for the contained decoder to advance as it wills. Cloning at
     // this point ensures it will begin at the end of the dir entries.
-    mReturnIterator = Move(mLexer.Clone(*mIterator, SIZE_MAX));
+    mReturnIterator = mLexer.Clone(*mIterator, SIZE_MAX);
     if (mReturnIterator.isNothing()) {
       // If we cannot read further than this point, then there is no resource
       // data to read.
@@ -223,7 +205,7 @@ nsICODecoder::IterateUnsizedDirEntry()
 
     // Our iterator is at an unknown point, so reset it to the point that we
     // saved.
-    mIterator = Move(mLexer.Clone(*mReturnIterator, SIZE_MAX));
+    mIterator = mLexer.Clone(*mReturnIterator, SIZE_MAX);
     if (mIterator.isNothing()) {
       MOZ_ASSERT_UNREACHABLE("Cannot re-clone return iterator");
       return Transition::TerminateFailure();
@@ -241,13 +223,10 @@ nsICODecoder::IterateUnsizedDirEntry()
   mDirEntry = &mUnsizedDirEntries[0];
   size_t offsetToResource = mDirEntry->mImageOffset - FirstResourceOffset();
   return Transition::ToUnbuffered(ICOState::FOUND_RESOURCE,
-                                  ICOState::SKIP_TO_RESOURCE,
-                                  offsetToResource);
+                                  ICOState::SKIP_TO_RESOURCE, offsetToResource);
 }
 
-LexerTransition<ICOState>
-nsICODecoder::FinishDirEntry()
-{
+LexerTransition<ICOState> nsICODecoder::FinishDirEntry() {
   MOZ_ASSERT(!mDirEntry);
 
   if (mDirEntries.IsEmpty()) {
@@ -271,7 +250,7 @@ nsICODecoder::FinishDirEntry()
     if (!biggestEntry ||
         (e.mBitCount >= biggestEntry->mBitCount &&
          e.mSize.width * e.mSize.height >=
-           biggestEntry->mSize.width * biggestEntry->mSize.height)) {
+             biggestEntry->mSize.width * biggestEntry->mSize.height)) {
       biggestEntry = &e;
 
       if (!desiredSize) {
@@ -280,20 +259,19 @@ nsICODecoder::FinishDirEntry()
     }
 
     if (desiredSize) {
-      // Calculate the delta between this resource's size and the desired size, so
-      // we can see if it is better than our current-best option.  In the case of
-      // several equally-good resources, we use the last one. "Better" in this
-      // case is determined by |delta|, a measure of the difference in size
-      // between the entry we've found and the desired size. We will choose the
-      // smallest resource that is greater than or equal to the desired size (i.e.
-      // we assume it's better to downscale a larger icon than to upscale a
-      // smaller one).
+      // Calculate the delta between this resource's size and the desired size,
+      // so we can see if it is better than our current-best option.  In the
+      // case of several equally-good resources, we use the last one. "Better"
+      // in this case is determined by |delta|, a measure of the difference in
+      // size between the entry we've found and the desired size. We will choose
+      // the smallest resource that is greater than or equal to the desired size
+      // (i.e. we assume it's better to downscale a larger icon than to upscale
+      // a smaller one).
       int32_t delta = std::min(e.mSize.width - desiredSize->width,
                                e.mSize.height - desiredSize->height);
-      if (!mDirEntry ||
-          (e.mBitCount >= mDirEntry->mBitCount &&
-           ((bestDelta < 0 && delta >= bestDelta) ||
-            (delta >= 0 && delta <= bestDelta)))) {
+      if (!mDirEntry || (e.mBitCount >= mDirEntry->mBitCount &&
+                         ((bestDelta < 0 && delta >= bestDelta) ||
+                          (delta >= 0 && delta <= bestDelta)))) {
         mDirEntry = &e;
         bestDelta = delta;
       }
@@ -306,8 +284,7 @@ nsICODecoder::FinishDirEntry()
   // If this is a cursor, set the hotspot. We use the hotspot from the biggest
   // resource since we also use that resource for the intrinsic size.
   if (mIsCursor) {
-    mImageMetadata.SetHotspot(biggestEntry->mXHotspot,
-                              biggestEntry->mYHotspot);
+    mImageMetadata.SetHotspot(biggestEntry->mXHotspot, biggestEntry->mYHotspot);
   }
 
   // We always report the biggest resource's size as the intrinsic size; this
@@ -332,13 +309,10 @@ nsICODecoder::FinishDirEntry()
 
   size_t offsetToResource = mDirEntry->mImageOffset - FirstResourceOffset();
   return Transition::ToUnbuffered(ICOState::FOUND_RESOURCE,
-                                  ICOState::SKIP_TO_RESOURCE,
-                                  offsetToResource);
+                                  ICOState::SKIP_TO_RESOURCE, offsetToResource);
 }
 
-LexerTransition<ICOState>
-nsICODecoder::SniffResource(const char* aData)
-{
+LexerTransition<ICOState> nsICODecoder::SniffResource(const char* aData) {
   MOZ_ASSERT(mDirEntry);
 
   // We have BITMAPINFOSIZE bytes buffered at this point. We know an embedded
@@ -352,8 +326,8 @@ nsICODecoder::SniffResource(const char* aData)
 
   // We use the first PNGSIGNATURESIZE bytes to determine whether this resource
   // is a PNG or a BMP.
-  bool isPNG = !memcmp(aData, nsPNGDecoder::pngSignatureBytes,
-                       PNGSIGNATURESIZE);
+  bool isPNG =
+      !memcmp(aData, nsPNGDecoder::pngSignatureBytes, PNGSIGNATURESIZE);
   if (isPNG) {
     if (mDirEntry->mBytesInRes <= BITMAPINFOSIZE) {
       return Transition::TerminateFailure();
@@ -361,28 +335,24 @@ nsICODecoder::SniffResource(const char* aData)
 
     // Prepare a new iterator for the contained decoder to advance as it wills.
     // Cloning at the point ensures it will begin at the resource offset.
-    Maybe<SourceBufferIterator> containedIterator
-      = mLexer.Clone(*mIterator, mDirEntry->mBytesInRes);
+    Maybe<SourceBufferIterator> containedIterator =
+        mLexer.Clone(*mIterator, mDirEntry->mBytesInRes);
     if (containedIterator.isNothing()) {
       return Transition::TerminateFailure();
     }
 
     // Create a PNG decoder which will do the rest of the work for us.
     bool metadataDecode = mReturnIterator.isSome();
-    Maybe<IntSize> expectedSize = metadataDecode ? Nothing()
-                                                 : Some(mDirEntry->mSize);
-    mContainedDecoder =
-      DecoderFactory::CreateDecoderForICOResource(DecoderType::PNG,
-                                                  Move(containedIterator.ref()),
-                                                  WrapNotNull(this),
-                                                  metadataDecode,
-                                                  expectedSize);
+    Maybe<IntSize> expectedSize =
+        metadataDecode ? Nothing() : Some(mDirEntry->mSize);
+    mContainedDecoder = DecoderFactory::CreateDecoderForICOResource(
+        DecoderType::PNG, std::move(containedIterator.ref()), WrapNotNull(this),
+        metadataDecode, expectedSize);
 
     // Read in the rest of the PNG unbuffered.
     size_t toRead = mDirEntry->mBytesInRes - BITMAPINFOSIZE;
     return Transition::ToUnbuffered(ICOState::FINISHED_RESOURCE,
-                                    ICOState::READ_RESOURCE,
-                                    toRead);
+                                    ICOState::READ_RESOURCE, toRead);
   } else {
     // Make sure we have a sane size for the bitmap information header.
     int32_t bihSize = LittleEndian::readUint32(aData);
@@ -395,9 +365,7 @@ nsICODecoder::SniffResource(const char* aData)
   }
 }
 
-LexerTransition<ICOState>
-nsICODecoder::ReadResource()
-{
+LexerTransition<ICOState> nsICODecoder::ReadResource() {
   if (!FlushContainedDecoder()) {
     return Transition::TerminateFailure();
   }
@@ -405,9 +373,7 @@ nsICODecoder::ReadResource()
   return Transition::ContinueUnbuffered(ICOState::READ_RESOURCE);
 }
 
-LexerTransition<ICOState>
-nsICODecoder::ReadBIH(const char* aData)
-{
+LexerTransition<ICOState> nsICODecoder::ReadBIH(const char* aData) {
   MOZ_ASSERT(mDirEntry);
 
   // Extract the BPP from the BIH header; it should be trusted over the one
@@ -427,12 +393,13 @@ nsICODecoder::ReadBIH(const char* aData)
   // bitmap file header. So we create the BMP decoder via the constructor that
   // tells it to skip this, and pass in the required data (dataOffset) that
   // would have been present in the header.
-  uint32_t dataOffset = bmp::FILE_HEADER_LENGTH + BITMAPINFOSIZE + 4 * numColors;
+  uint32_t dataOffset =
+      bmp::FILE_HEADER_LENGTH + BITMAPINFOSIZE + 4 * numColors;
 
   // Prepare a new iterator for the contained decoder to advance as it wills.
   // Cloning at the point ensures it will begin at the resource offset.
-  Maybe<SourceBufferIterator> containedIterator
-    = mLexer.Clone(*mIterator, mDirEntry->mBytesInRes);
+  Maybe<SourceBufferIterator> containedIterator =
+      mLexer.Clone(*mIterator, mDirEntry->mBytesInRes);
   if (containedIterator.isNothing()) {
     return Transition::TerminateFailure();
   }
@@ -440,18 +407,14 @@ nsICODecoder::ReadBIH(const char* aData)
   // Create a BMP decoder which will do most of the work for us; the exception
   // is the AND mask, which isn't present in standalone BMPs.
   bool metadataDecode = mReturnIterator.isSome();
-  Maybe<IntSize> expectedSize = metadataDecode ? Nothing()
-                                               : Some(mDirEntry->mSize);
-  mContainedDecoder =
-    DecoderFactory::CreateDecoderForICOResource(DecoderType::BMP,
-                                                Move(containedIterator.ref()),
-                                                WrapNotNull(this),
-                                                metadataDecode,
-                                                expectedSize,
-                                                Some(dataOffset));
+  Maybe<IntSize> expectedSize =
+      metadataDecode ? Nothing() : Some(mDirEntry->mSize);
+  mContainedDecoder = DecoderFactory::CreateDecoderForICOResource(
+      DecoderType::BMP, std::move(containedIterator.ref()), WrapNotNull(this),
+      metadataDecode, expectedSize, Some(dataOffset));
 
   RefPtr<nsBMPDecoder> bmpDecoder =
-    static_cast<nsBMPDecoder*>(mContainedDecoder.get());
+      static_cast<nsBMPDecoder*>(mContainedDecoder.get());
 
   // Ensure the decoder has parsed at least the BMP's bitmap info header.
   if (!FlushContainedDecoder()) {
@@ -467,18 +430,15 @@ nsICODecoder::ReadBIH(const char* aData)
   // the BMP data itself.
   uint32_t bmpDataLength = bmpDecoder->GetCompressedImageSize() + 4 * numColors;
   bool hasANDMask = (BITMAPINFOSIZE + bmpDataLength) < mDirEntry->mBytesInRes;
-  ICOState afterBMPState = hasANDMask ? ICOState::PREPARE_FOR_MASK
-                                      : ICOState::FINISHED_RESOURCE;
+  ICOState afterBMPState =
+      hasANDMask ? ICOState::PREPARE_FOR_MASK : ICOState::FINISHED_RESOURCE;
 
   // Read in the rest of the BMP unbuffered.
-  return Transition::ToUnbuffered(afterBMPState,
-                                  ICOState::READ_RESOURCE,
+  return Transition::ToUnbuffered(afterBMPState, ICOState::READ_RESOURCE,
                                   bmpDataLength);
 }
 
-LexerTransition<ICOState>
-nsICODecoder::PrepareForMask()
-{
+LexerTransition<ICOState> nsICODecoder::PrepareForMask() {
   MOZ_ASSERT(mDirEntry);
   MOZ_ASSERT(mContainedDecoder->GetDecodeDone());
 
@@ -491,26 +451,25 @@ nsICODecoder::PrepareForMask()
   MOZ_ASSERT(mContainedDecoder->GetDecodeDone());
 
   RefPtr<nsBMPDecoder> bmpDecoder =
-    static_cast<nsBMPDecoder*>(mContainedDecoder.get());
+      static_cast<nsBMPDecoder*>(mContainedDecoder.get());
 
   uint16_t numColors = GetNumColors();
   MOZ_ASSERT(numColors != uint16_t(-1));
 
   // Determine the length of the AND mask.
   uint32_t bmpLengthWithHeader =
-    BITMAPINFOSIZE + bmpDecoder->GetCompressedImageSize() + 4 * numColors;
+      BITMAPINFOSIZE + bmpDecoder->GetCompressedImageSize() + 4 * numColors;
   MOZ_ASSERT(bmpLengthWithHeader < mDirEntry->mBytesInRes);
   uint32_t maskLength = mDirEntry->mBytesInRes - bmpLengthWithHeader;
 
   // If the BMP provides its own transparency, we ignore the AND mask.
   if (bmpDecoder->HasTransparency()) {
     return Transition::ToUnbuffered(ICOState::FINISHED_RESOURCE,
-                                    ICOState::SKIP_MASK,
-                                    maskLength);
+                                    ICOState::SKIP_MASK, maskLength);
   }
 
   // Compute the row size for the mask.
-  mMaskRowSize = ((mDirEntry->mSize.width + 31) / 32) * 4; // + 31 to round up
+  mMaskRowSize = ((mDirEntry->mSize.width + 31) / 32) * 4;  // + 31 to round up
 
   // If the expected size of the AND mask is larger than its actual size, then
   // we must have a truncated (and therefore corrupt) AND mask.
@@ -524,14 +483,13 @@ nsICODecoder::PrepareForMask()
   // combine the mask's alpha values with the color values from the image.
   if (mDownscaler) {
     MOZ_ASSERT(bmpDecoder->GetImageDataLength() ==
-                 mDownscaler->TargetSize().width *
-                 mDownscaler->TargetSize().height *
-                 sizeof(uint32_t));
+               mDownscaler->TargetSize().width *
+                   mDownscaler->TargetSize().height * sizeof(uint32_t));
     mMaskBuffer = MakeUnique<uint8_t[]>(bmpDecoder->GetImageDataLength());
-    nsresult rv = mDownscaler->BeginFrame(mDirEntry->mSize, Nothing(),
-                                          mMaskBuffer.get(),
-                                          /* aHasAlpha = */ true,
-                                          /* aFlipVertically = */ true);
+    nsresult rv =
+        mDownscaler->BeginFrame(mDirEntry->mSize, Nothing(), mMaskBuffer.get(),
+                                /* aHasAlpha = */ true,
+                                /* aFlipVertically = */ true);
     if (NS_FAILED(rv)) {
       return Transition::TerminateFailure();
     }
@@ -541,10 +499,7 @@ nsICODecoder::PrepareForMask()
   return Transition::To(ICOState::READ_MASK_ROW, mMaskRowSize);
 }
 
-
-LexerTransition<ICOState>
-nsICODecoder::ReadMaskRow(const char* aData)
-{
+LexerTransition<ICOState> nsICODecoder::ReadMaskRow(const char* aData) {
   MOZ_ASSERT(mDirEntry);
 
   mCurrMaskLine--;
@@ -560,12 +515,13 @@ nsICODecoder::ReadMaskRow(const char* aData)
   uint32_t* decoded = nullptr;
   if (mDownscaler) {
     // Initialize the row to all white and fully opaque.
-    memset(mDownscaler->RowBuffer(), 0xFF, mDirEntry->mSize.width * sizeof(uint32_t));
+    memset(mDownscaler->RowBuffer(), 0xFF,
+           mDirEntry->mSize.width * sizeof(uint32_t));
 
     decoded = reinterpret_cast<uint32_t*>(mDownscaler->RowBuffer());
   } else {
     RefPtr<nsBMPDecoder> bmpDecoder =
-      static_cast<nsBMPDecoder*>(mContainedDecoder.get());
+        static_cast<nsBMPDecoder*>(mContainedDecoder.get());
     uint32_t* imageData = bmpDecoder->GetImageData();
     if (!imageData) {
       return Transition::TerminateFailure();
@@ -607,15 +563,13 @@ nsICODecoder::ReadMaskRow(const char* aData)
   return Transition::To(ICOState::READ_MASK_ROW, mMaskRowSize);
 }
 
-LexerTransition<ICOState>
-nsICODecoder::FinishMask()
-{
+LexerTransition<ICOState> nsICODecoder::FinishMask() {
   // If we're downscaling, we now have the appropriate alpha values in
   // mMaskBuffer. We just need to transfer them to the image.
   if (mDownscaler) {
     // Retrieve the image data.
     RefPtr<nsBMPDecoder> bmpDecoder =
-      static_cast<nsBMPDecoder*>(mContainedDecoder.get());
+        static_cast<nsBMPDecoder*>(mContainedDecoder.get());
     uint8_t* imageData = reinterpret_cast<uint8_t*>(bmpDecoder->GetImageData());
     if (!imageData) {
       return Transition::TerminateFailure();
@@ -624,24 +578,23 @@ nsICODecoder::FinishMask()
     // Iterate through the alpha values, copying from mask to image.
     MOZ_ASSERT(mMaskBuffer);
     MOZ_ASSERT(bmpDecoder->GetImageDataLength() > 0);
-    for (size_t i = 3 ; i < bmpDecoder->GetImageDataLength() ; i += 4) {
+    for (size_t i = 3; i < bmpDecoder->GetImageDataLength(); i += 4) {
       imageData[i] = mMaskBuffer[i];
     }
     int32_t stride = mDownscaler->TargetSize().width * sizeof(uint32_t);
     DebugOnly<bool> ret =
-    // We know the format is B8G8R8A8 because we always assume bmp's inside
-    // ico's are transparent. 
-      PremultiplyData(imageData, stride, SurfaceFormat::B8G8R8A8,
-        imageData, stride, SurfaceFormat::B8G8R8A8, mDownscaler->TargetSize());
+        // We know the format is B8G8R8A8 because we always assume bmp's inside
+        // ico's are transparent.
+        PremultiplyData(imageData, stride, SurfaceFormat::B8G8R8A8, imageData,
+                        stride, SurfaceFormat::B8G8R8A8,
+                        mDownscaler->TargetSize());
     MOZ_ASSERT(ret);
   }
 
   return Transition::To(ICOState::FINISHED_RESOURCE, 0);
 }
 
-LexerTransition<ICOState>
-nsICODecoder::FinishResource()
-{
+LexerTransition<ICOState> nsICODecoder::FinishResource() {
   MOZ_ASSERT(mDirEntry);
 
   // We have received all of the data required by the PNG/BMP decoder so
@@ -674,49 +627,47 @@ nsICODecoder::FinishResource()
   return Transition::TerminateSuccess();
 }
 
-LexerResult
-nsICODecoder::DoDecode(SourceBufferIterator& aIterator, IResumable* aOnResume)
-{
+LexerResult nsICODecoder::DoDecode(SourceBufferIterator& aIterator,
+                                   IResumable* aOnResume) {
   MOZ_ASSERT(!HasError(), "Shouldn't call DoDecode after error!");
 
-  return mLexer.Lex(aIterator, aOnResume,
-                    [=](ICOState aState, const char* aData, size_t aLength) {
-    switch (aState) {
-      case ICOState::HEADER:
-        return ReadHeader(aData);
-      case ICOState::DIR_ENTRY:
-        return ReadDirEntry(aData);
-      case ICOState::FINISHED_DIR_ENTRY:
-        return FinishDirEntry();
-      case ICOState::ITERATE_UNSIZED_DIR_ENTRY:
-        return IterateUnsizedDirEntry();
-      case ICOState::SKIP_TO_RESOURCE:
-        return Transition::ContinueUnbuffered(ICOState::SKIP_TO_RESOURCE);
-      case ICOState::FOUND_RESOURCE:
-        return Transition::To(ICOState::SNIFF_RESOURCE, BITMAPINFOSIZE);
-      case ICOState::SNIFF_RESOURCE:
-        return SniffResource(aData);
-      case ICOState::READ_RESOURCE:
-        return ReadResource();
-      case ICOState::PREPARE_FOR_MASK:
-        return PrepareForMask();
-      case ICOState::READ_MASK_ROW:
-        return ReadMaskRow(aData);
-      case ICOState::FINISH_MASK:
-        return FinishMask();
-      case ICOState::SKIP_MASK:
-        return Transition::ContinueUnbuffered(ICOState::SKIP_MASK);
-      case ICOState::FINISHED_RESOURCE:
-        return FinishResource();
-      default:
-        MOZ_CRASH("Unknown ICOState");
-    }
-  });
+  return mLexer.Lex(
+      aIterator, aOnResume,
+      [=](ICOState aState, const char* aData, size_t aLength) {
+        switch (aState) {
+          case ICOState::HEADER:
+            return ReadHeader(aData);
+          case ICOState::DIR_ENTRY:
+            return ReadDirEntry(aData);
+          case ICOState::FINISHED_DIR_ENTRY:
+            return FinishDirEntry();
+          case ICOState::ITERATE_UNSIZED_DIR_ENTRY:
+            return IterateUnsizedDirEntry();
+          case ICOState::SKIP_TO_RESOURCE:
+            return Transition::ContinueUnbuffered(ICOState::SKIP_TO_RESOURCE);
+          case ICOState::FOUND_RESOURCE:
+            return Transition::To(ICOState::SNIFF_RESOURCE, BITMAPINFOSIZE);
+          case ICOState::SNIFF_RESOURCE:
+            return SniffResource(aData);
+          case ICOState::READ_RESOURCE:
+            return ReadResource();
+          case ICOState::PREPARE_FOR_MASK:
+            return PrepareForMask();
+          case ICOState::READ_MASK_ROW:
+            return ReadMaskRow(aData);
+          case ICOState::FINISH_MASK:
+            return FinishMask();
+          case ICOState::SKIP_MASK:
+            return Transition::ContinueUnbuffered(ICOState::SKIP_MASK);
+          case ICOState::FINISHED_RESOURCE:
+            return FinishResource();
+          default:
+            MOZ_CRASH("Unknown ICOState");
+        }
+      });
 }
 
-bool
-nsICODecoder::FlushContainedDecoder()
-{
+bool nsICODecoder::FlushContainedDecoder() {
   MOZ_ASSERT(mContainedDecoder);
 
   bool succeeded = true;
@@ -744,5 +695,5 @@ nsICODecoder::FlushContainedDecoder()
   return succeeded;
 }
 
-} // namespace image
-} // namespace mozilla
+}  // namespace image
+}  // namespace mozilla
