@@ -16,18 +16,19 @@
 #include "mozilla/gfx/PathHelpers.h"
 #include "mozilla/layers/LayersMessages.h"
 #include "mozilla/layers/StackingContextHelper.h"
-#include "mozilla/layers/WebRenderLayerManager.h"
+#include "mozilla/layers/RenderRootStateManager.h"
 #include "mozilla/layers/WebRenderMessages.h"
 #include "mozilla/MathAlgorithms.h"
 #include "mozilla/Move.h"
+#include "mozilla/PresShell.h"
 #include "nsCOMPtr.h"
+#include "nsCSSFrameConstructor.h"
 #include "nsFontMetrics.h"
 #include "nsGkAtoms.h"
 #include "nsGenericHTMLElement.h"
 #include "nsAttrValueInlines.h"
 #include "nsPresContext.h"
-#include "nsIPresShell.h"
-#include "nsIDocument.h"
+#include "mozilla/dom/Document.h"
 #include "nsDisplayList.h"
 #include "nsCounterManager.h"
 #include "nsBidiUtils.h"
@@ -45,13 +46,18 @@
 #include <algorithm>
 
 #ifdef ACCESSIBILITY
-#include "nsAccessibilityService.h"
+#  include "nsAccessibilityService.h"
 #endif
 
 using namespace mozilla;
 using namespace mozilla::gfx;
 using namespace mozilla::image;
 using namespace mozilla::layout;
+using mozilla::dom::Document;
+
+nsIFrame* NS_NewBulletFrame(PresShell* aPresShell, ComputedStyle* aStyle) {
+  return new (aPresShell) nsBulletFrame(aStyle, aPresShell->GetPresContext());
+}
 
 NS_DECLARE_FRAME_PROPERTY_SMALL_VALUE(FontSizeInflationProperty, float)
 
@@ -59,12 +65,17 @@ NS_IMPL_FRAMEARENA_HELPERS(nsBulletFrame)
 
 #ifdef DEBUG
 NS_QUERYFRAME_HEAD(nsBulletFrame)
-NS_QUERYFRAME_ENTRY(nsBulletFrame)
+  NS_QUERYFRAME_ENTRY(nsBulletFrame)
 NS_QUERYFRAME_TAIL_INHERITING(nsFrame)
 #endif
 
 nsBulletFrame::~nsBulletFrame() {}
 
+CounterStyle* nsBulletFrame::ResolveCounterStyle() {
+  return PresContext()->CounterStyleManager()->ResolveCounterStyle(
+      StyleList()->mCounterStyle);
+}
+
 void nsBulletFrame::DestroyFrom(nsIFrame* aDestructRoot,
                                 PostDestroyData& aPostDestroyData) {
   // Stop image loading first.
@@ -87,12 +98,12 @@ nsresult nsBulletFrame::GetFrameName(nsAString& aResult) const {
 bool nsBulletFrame::IsEmpty() { return IsSelfEmpty(); }
 
 bool nsBulletFrame::IsSelfEmpty() {
-  return StyleList()->mCounterStyle->IsNone();
+  return StyleList()->mCounterStyle.IsNone();
 }
 
-/* virtual */ void nsBulletFrame::DidSetStyleContext(
-    nsStyleContext* aOldStyleContext) {
-  nsFrame::DidSetStyleContext(aOldStyleContext);
+/* virtual */
+void nsBulletFrame::DidSetComputedStyle(ComputedStyle* aOldComputedStyle) {
+  nsFrame::DidSetComputedStyle(aOldComputedStyle);
 
   imgRequestProxy* newRequest = StyleList()->GetListStyleImage();
 
@@ -130,37 +141,40 @@ bool nsBulletFrame::IsSelfEmpty() {
       DeregisterAndCancelImageRequest();
 
       // Register the new request.
-      mImageRequest = Move(newRequestClone);
+      mImageRequest = std::move(newRequestClone);
       RegisterImageRequest(/* aKnownToBeAnimated = */ false);
+
+      // Image bullets can affect the layout of the page, so boost the image
+      // load priority.
+      mImageRequest->BoostPriority(imgIRequest::CATEGORY_SIZE_QUERY);
     }
   } else {
-    // No image request on the new style context.
+    // No image request on the new ComputedStyle.
     DeregisterAndCancelImageRequest();
   }
 
 #ifdef ACCESSIBILITY
   // Update the list bullet accessible. If old style list isn't available then
   // no need to update the accessible tree because it's not created yet.
-  if (aOldStyleContext) {
-    nsAccessibilityService* accService = nsIPresShell::AccService();
-    if (accService) {
-      const nsStyleList* oldStyleList = aOldStyleContext->PeekStyleList();
-      if (oldStyleList) {
-        bool hadBullet = oldStyleList->GetListStyleImage() ||
-                         !oldStyleList->mCounterStyle->IsNone();
+  if (aOldComputedStyle) {
+    if (nsAccessibilityService* accService =
+            PresShell::GetAccessibilityService()) {
+      const nsStyleList* oldStyleList = aOldComputedStyle->StyleList();
+      bool hadBullet = oldStyleList->GetListStyleImage() ||
+                       !oldStyleList->mCounterStyle.IsNone();
 
-        const nsStyleList* newStyleList = StyleList();
-        bool hasBullet = newStyleList->GetListStyleImage() ||
-                         !newStyleList->mCounterStyle->IsNone();
+      const nsStyleList* newStyleList = StyleList();
+      bool hasBullet = newStyleList->GetListStyleImage() ||
+                       !newStyleList->mCounterStyle.IsNone();
 
-        if (hadBullet != hasBullet) {
-          accService->UpdateListBullet(PresContext()->GetPresShell(), mContent,
-                                       hasBullet);
-        }
+      if (hadBullet != hasBullet) {
+        nsIContent* listItem = mContent->GetParent();
+        accService->UpdateListBullet(PresContext()->GetPresShell(), listItem,
+                                     hasBullet);
       }
     }
   }
-#endif
+#endif  // #ifdef ACCESSIBILITY
 }
 
 class nsDisplayBulletGeometry
@@ -171,7 +185,7 @@ class nsDisplayBulletGeometry
       : nsDisplayItemGenericGeometry(aItem, aBuilder),
         nsImageGeometryMixin(aItem, aBuilder) {
     nsBulletFrame* f = static_cast<nsBulletFrame*>(aItem->Frame());
-    mOrdinal = f->GetOrdinal();
+    mOrdinal = f->Ordinal();
   }
 
   virtual bool InvalidateForSyncDecodeImages() const override {
@@ -184,7 +198,10 @@ class nsDisplayBulletGeometry
 class BulletRenderer final {
  public:
   BulletRenderer(imgIContainer* image, const nsRect& dest)
-      : mImage(image), mDest(dest), mListStyleType(NS_STYLE_LIST_STYLE_NONE) {
+      : mImage(image),
+        mDest(dest),
+        mColor(NS_RGBA(0, 0, 0, 0)),
+        mListStyleType(NS_STYLE_LIST_STYLE_NONE) {
     MOZ_ASSERT(IsImageType());
   }
 
@@ -209,12 +226,12 @@ class BulletRenderer final {
     MOZ_ASSERT(IsTextType());
   }
 
-  bool CreateWebRenderCommands(nsDisplayItem* aItem,
-                               wr::DisplayListBuilder& aBuilder,
-                               wr::IpcResourceUpdateQueue& aResources,
-                               const layers::StackingContextHelper& aSc,
-                               mozilla::layers::WebRenderLayerManager* aManager,
-                               nsDisplayListBuilder* aDisplayListBuilder);
+  ImgDrawResult CreateWebRenderCommands(
+      nsDisplayItem* aItem, wr::DisplayListBuilder& aBuilder,
+      wr::IpcResourceUpdateQueue& aResources,
+      const layers::StackingContextHelper& aSc,
+      mozilla::layers::RenderRootStateManager* aManager,
+      nsDisplayListBuilder* aDisplayListBuilder);
 
   ImgDrawResult Paint(gfxContext& aRenderingContext, nsPoint aPt,
                       const nsRect& aDirtyRect, uint32_t aFlags,
@@ -242,8 +259,6 @@ class BulletRenderer final {
            !mText.IsEmpty();
   }
 
-  bool BuildGlyphForText(nsDisplayItem* aItem, bool disableSubpixelAA);
-
   void PaintTextToContext(nsIFrame* aFrame, gfxContext* aCtx,
                           bool aDisableSubpixelAA);
 
@@ -251,25 +266,25 @@ class BulletRenderer final {
                                  uint32_t aFlags);
 
  private:
-  bool CreateWebRenderCommandsForImage(
+  ImgDrawResult CreateWebRenderCommandsForImage(
       nsDisplayItem* aItem, wr::DisplayListBuilder& aBuilder,
       wr::IpcResourceUpdateQueue& aResources,
       const layers::StackingContextHelper& aSc,
-      mozilla::layers::WebRenderLayerManager* aManager,
+      mozilla::layers::RenderRootStateManager* aManager,
       nsDisplayListBuilder* aDisplayListBuilder);
 
   bool CreateWebRenderCommandsForPath(
       nsDisplayItem* aItem, wr::DisplayListBuilder& aBuilder,
       wr::IpcResourceUpdateQueue& aResources,
       const layers::StackingContextHelper& aSc,
-      mozilla::layers::WebRenderLayerManager* aManager,
+      mozilla::layers::RenderRootStateManager* aManager,
       nsDisplayListBuilder* aDisplayListBuilder);
 
   bool CreateWebRenderCommandsForText(
       nsDisplayItem* aItem, wr::DisplayListBuilder& aBuilder,
       wr::IpcResourceUpdateQueue& aResources,
       const layers::StackingContextHelper& aSc,
-      mozilla::layers::WebRenderLayerManager* aManager,
+      mozilla::layers::RenderRootStateManager* aManager,
       nsDisplayListBuilder* aDisplayListBuilder);
 
  private:
@@ -288,7 +303,7 @@ class BulletRenderer final {
   LayoutDeviceRect mPathRect;
 
   // mColor indicate the color of list-style. Both text and path type would use
-  // this memeber.
+  // this member.
   nscolor mColor;
 
   // mPath record the path of the list-style for later drawing.
@@ -308,23 +323,28 @@ class BulletRenderer final {
   int32_t mListStyleType;
 };
 
-bool BulletRenderer::CreateWebRenderCommands(
+ImgDrawResult BulletRenderer::CreateWebRenderCommands(
     nsDisplayItem* aItem, wr::DisplayListBuilder& aBuilder,
     wr::IpcResourceUpdateQueue& aResources,
     const layers::StackingContextHelper& aSc,
-    mozilla::layers::WebRenderLayerManager* aManager,
+    mozilla::layers::RenderRootStateManager* aManager,
     nsDisplayListBuilder* aDisplayListBuilder) {
   if (IsImageType()) {
     return CreateWebRenderCommandsForImage(aItem, aBuilder, aResources, aSc,
                                            aManager, aDisplayListBuilder);
-  } else if (IsPathType()) {
-    return CreateWebRenderCommandsForPath(aItem, aBuilder, aResources, aSc,
-                                          aManager, aDisplayListBuilder);
+  }
+
+  bool success;
+  if (IsPathType()) {
+    success = CreateWebRenderCommandsForPath(aItem, aBuilder, aResources, aSc,
+                                             aManager, aDisplayListBuilder);
   } else {
     MOZ_ASSERT(IsTextType());
-    return CreateWebRenderCommandsForText(aItem, aBuilder, aResources, aSc,
-                                          aManager, aDisplayListBuilder);
+    success = CreateWebRenderCommandsForText(aItem, aBuilder, aResources, aSc,
+                                             aManager, aDisplayListBuilder);
   }
+
+  return success ? ImgDrawResult::SUCCESS : ImgDrawResult::NOT_SUPPORTED;
 }
 
 ImgDrawResult BulletRenderer::Paint(gfxContext& aRenderingContext, nsPoint aPt,
@@ -380,35 +400,6 @@ ImgDrawResult BulletRenderer::Paint(gfxContext& aRenderingContext, nsPoint aPt,
   return ImgDrawResult::SUCCESS;
 }
 
-bool BulletRenderer::BuildGlyphForText(nsDisplayItem* aItem,
-                                       bool disableSubpixelAA) {
-  MOZ_ASSERT(IsTextType());
-
-  RefPtr<DrawTarget> screenTarget =
-      gfxPlatform::GetPlatform()->ScreenReferenceDrawTarget();
-  RefPtr<DrawTargetCapture> capture = Factory::CreateCaptureDrawTarget(
-      screenTarget->GetBackendType(), IntSize(), screenTarget->GetFormat());
-
-  RefPtr<gfxContext> captureCtx = gfxContext::CreateOrNull(capture);
-
-  PaintTextToContext(aItem->Frame(), captureCtx, disableSubpixelAA);
-
-  layers::GlyphArray* g = mGlyphs.AppendElement();
-  std::vector<Glyph> glyphs;
-  Color color;
-  if (!capture->ContainsOnlyColoredGlyphs(mFont, color, glyphs)) {
-    mFont = nullptr;
-    mGlyphs.Clear();
-    return false;
-  }
-
-  g->glyphs().SetLength(glyphs.size());
-  PodCopy(g->glyphs().Elements(), glyphs.data(), glyphs.size());
-  g->color() = color;
-
-  return true;
-}
-
 void BulletRenderer::PaintTextToContext(nsIFrame* aFrame, gfxContext* aCtx,
                                         bool aDisableSubpixelAA) {
   MOZ_ASSERT(IsTextType());
@@ -434,11 +425,11 @@ bool BulletRenderer::IsImageContainerAvailable(layers::LayerManager* aManager,
   return mImage->IsImageContainerAvailable(aManager, aFlags);
 }
 
-bool BulletRenderer::CreateWebRenderCommandsForImage(
+ImgDrawResult BulletRenderer::CreateWebRenderCommandsForImage(
     nsDisplayItem* aItem, wr::DisplayListBuilder& aBuilder,
     wr::IpcResourceUpdateQueue& aResources,
     const layers::StackingContextHelper& aSc,
-    mozilla::layers::WebRenderLayerManager* aManager,
+    mozilla::layers::RenderRootStateManager* aManager,
     nsDisplayListBuilder* aDisplayListBuilder) {
   MOZ_RELEASE_ASSERT(IsImageType());
   MOZ_RELEASE_ASSERT(mImage);
@@ -455,39 +446,46 @@ bool BulletRenderer::CreateWebRenderCommandsForImage(
       aItem->Frame()->PresContext()->AppUnitsPerDevPixel();
   LayoutDeviceRect destRect =
       LayoutDeviceRect::FromAppUnits(mDest, appUnitsPerDevPixel);
+  destRect.Round();
+
   Maybe<SVGImageContext> svgContext;
   gfx::IntSize decodeSize =
       nsLayoutUtils::ComputeImageContainerDrawingParameters(
           mImage, aItem->Frame(), destRect, aSc, flags, svgContext);
-  RefPtr<layers::ImageContainer> container =
-      mImage->GetImageContainerAtSize(aManager, decodeSize, svgContext, flags);
+
+  RefPtr<layers::ImageContainer> container;
+  ImgDrawResult drawResult = mImage->GetImageContainerAtSize(
+      aManager->LayerManager(), decodeSize, svgContext, flags,
+      getter_AddRefs(container));
   if (!container) {
-    return false;
+    return drawResult;
   }
 
+  mozilla::wr::ImageRendering rendering = wr::ToImageRendering(
+      nsLayoutUtils::GetSamplingFilterForFrame(aItem->Frame()));
   gfx::IntSize size;
   Maybe<wr::ImageKey> key = aManager->CommandBuilder().CreateImageKey(
-      aItem, container, aBuilder, aResources, aSc, size, Nothing());
+      aItem, container, aBuilder, aResources, rendering, aSc, size, Nothing());
   if (key.isNothing()) {
-    return true;  // Nothing to do
+    return drawResult;
   }
 
-  wr::LayoutRect dest = aSc.ToRelativeLayoutRect(destRect);
+  wr::LayoutRect dest = wr::ToLayoutRect(destRect);
 
-  aBuilder.PushImage(dest, dest, !aItem->BackfaceIsHidden(),
-                     wr::ImageRendering::Auto, key.value());
+  aBuilder.PushImage(dest, dest, !aItem->BackfaceIsHidden(), rendering,
+                     key.value());
 
-  return true;
+  return drawResult;
 }
 
 bool BulletRenderer::CreateWebRenderCommandsForPath(
     nsDisplayItem* aItem, wr::DisplayListBuilder& aBuilder,
     wr::IpcResourceUpdateQueue& aResources,
     const layers::StackingContextHelper& aSc,
-    mozilla::layers::WebRenderLayerManager* aManager,
+    mozilla::layers::RenderRootStateManager* aManager,
     nsDisplayListBuilder* aDisplayListBuilder) {
   MOZ_ASSERT(IsPathType());
-  wr::LayoutRect dest = aSc.ToRelativeLayoutRect(mPathRect);
+  wr::LayoutRect dest = wr::ToRoundedLayoutRect(mPathRect);
   auto color = wr::ToColorF(ToDeviceColor(mColor));
   bool isBackfaceVisible = !aItem->BackfaceIsHidden();
   switch (mListStyleType) {
@@ -503,15 +501,7 @@ bool BulletRenderer::CreateWebRenderCommandsForPath(
       return true;
     }
     case NS_STYLE_LIST_STYLE_DISC: {
-      nsTArray<wr::ComplexClipRegion> clips;
-      clips.AppendElement(wr::ToComplexClipRegion(
-          RoundedRect(ThebesRect(mPathRect.ToUnknownRect()),
-                      RectCornerRadii(dest.size.width / 2.0))));
-      auto clipId =
-          aBuilder.DefineClip(Nothing(), Nothing(), dest, &clips, nullptr);
-      aBuilder.PushClip(clipId);
-      aBuilder.PushRect(dest, dest, isBackfaceVisible, color);
-      aBuilder.PopClip();
+      aBuilder.PushRoundedRect(dest, dest, isBackfaceVisible, color);
       return true;
     }
     case NS_STYLE_LIST_STYLE_SQUARE: {
@@ -533,7 +523,7 @@ bool BulletRenderer::CreateWebRenderCommandsForText(
     nsDisplayItem* aItem, wr::DisplayListBuilder& aBuilder,
     wr::IpcResourceUpdateQueue& aResources,
     const layers::StackingContextHelper& aSc,
-    mozilla::layers::WebRenderLayerManager* aManager,
+    mozilla::layers::RenderRootStateManager* aManager,
     nsDisplayListBuilder* aDisplayListBuilder) {
   MOZ_ASSERT(IsTextType());
 
@@ -545,18 +535,18 @@ bool BulletRenderer::CreateWebRenderCommandsForText(
   }
 
   RefPtr<TextDrawTarget> textDrawer =
-      new TextDrawTarget(aBuilder, aSc, aManager, aItem, bounds);
+      new TextDrawTarget(aBuilder, aResources, aSc, aManager, aItem, bounds);
   RefPtr<gfxContext> captureCtx = gfxContext::CreateOrNull(textDrawer);
   PaintTextToContext(aItem->Frame(), captureCtx, aItem->IsSubpixelAADisabled());
   textDrawer->TerminateShadows();
 
-  return !textDrawer->HasUnsupportedFeatures();
+  return textDrawer->Finish();
 }
 
-class nsDisplayBullet final : public nsDisplayItem {
+class nsDisplayBullet final : public nsPaintedDisplayItem {
  public:
   nsDisplayBullet(nsDisplayListBuilder* aBuilder, nsBulletFrame* aFrame)
-      : nsDisplayItem(aBuilder, aFrame) {
+      : nsPaintedDisplayItem(aBuilder, aFrame) {
     MOZ_COUNT_CTOR(nsDisplayBullet);
   }
 #ifdef NS_BUILD_REFCNT_LOGGING
@@ -569,18 +559,10 @@ class nsDisplayBullet final : public nsDisplayItem {
     return mFrame->GetVisualOverflowRectRelativeToSelf() + ToReferenceFrame();
   }
 
-  virtual LayerState GetLayerState(
-      nsDisplayListBuilder* aBuilder, LayerManager* aManager,
-      const ContainerLayerParameters& aParameters) override;
-
-  virtual already_AddRefed<Layer> BuildLayer(
-      nsDisplayListBuilder* aBuilder, LayerManager* aManager,
-      const ContainerLayerParameters& aParameters) override;
-
   virtual bool CreateWebRenderCommands(
       mozilla::wr::DisplayListBuilder& aBuilder,
       mozilla::wr::IpcResourceUpdateQueue&, const StackingContextHelper& aSc,
-      mozilla::layers::WebRenderLayerManager* aManager,
+      mozilla::layers::RenderRootStateManager* aManager,
       nsDisplayListBuilder* aDisplayListBuilder) override;
 
   virtual void HitTest(nsDisplayListBuilder* aBuilder, const nsRect& aRect,
@@ -609,7 +591,7 @@ class nsDisplayBullet final : public nsDisplayItem {
         static_cast<const nsDisplayBulletGeometry*>(aGeometry);
     nsBulletFrame* f = static_cast<nsBulletFrame*>(mFrame);
 
-    if (f->GetOrdinal() != geometry->mOrdinal) {
+    if (f->Ordinal() != geometry->mOrdinal) {
       bool snap;
       aInvalidRegion->Or(geometry->mBounds, GetBounds(aBuilder, &snap));
       return;
@@ -622,65 +604,18 @@ class nsDisplayBullet final : public nsDisplayItem {
       aInvalidRegion->Or(*aInvalidRegion, GetBounds(aBuilder, &snap));
     }
 
-    return nsDisplayItem::ComputeInvalidationRegion(aBuilder, aGeometry,
-                                                    aInvalidRegion);
+    return nsPaintedDisplayItem::ComputeInvalidationRegion(aBuilder, aGeometry,
+                                                           aInvalidRegion);
   }
 
  protected:
   Maybe<BulletRenderer> mBulletRenderer;
 };
 
-LayerState nsDisplayBullet::GetLayerState(
-    nsDisplayListBuilder* aBuilder, LayerManager* aManager,
-    const ContainerLayerParameters& aParameters) {
-  if (!ShouldUseAdvancedLayer(aManager, gfxPrefs::LayersAllowBulletLayers)) {
-    return LAYER_NONE;
-  }
-  RefPtr<gfxContext> screenRefCtx = gfxContext::CreateOrNull(
-      gfxPlatform::GetPlatform()->ScreenReferenceDrawTarget().get());
-
-  Maybe<BulletRenderer> br =
-      static_cast<nsBulletFrame*>(mFrame)->CreateBulletRenderer(
-          *screenRefCtx, ToReferenceFrame());
-
-  if (!br) {
-    return LAYER_NONE;
-  }
-
-  if (br->IsImageType()) {
-    uint32_t flags = aBuilder->ShouldSyncDecodeImages()
-                         ? imgIContainer::FLAG_SYNC_DECODE
-                         : imgIContainer::FLAG_NONE;
-
-    if (!br->IsImageContainerAvailable(aManager, flags)) {
-      return LAYER_NONE;
-    }
-  }
-
-  if (br->IsTextType()) {
-    if (!br->BuildGlyphForText(this, mDisableSubpixelAA)) {
-      return LAYER_NONE;
-    }
-  }
-
-  mBulletRenderer = br;
-  return LAYER_ACTIVE;
-}
-
-already_AddRefed<layers::Layer> nsDisplayBullet::BuildLayer(
-    nsDisplayListBuilder* aBuilder, LayerManager* aManager,
-    const ContainerLayerParameters& aContainerParameters) {
-  if (!mBulletRenderer) {
-    return nullptr;
-  }
-
-  return BuildDisplayItemLayer(aBuilder, aManager, aContainerParameters);
-}
-
 bool nsDisplayBullet::CreateWebRenderCommands(
     wr::DisplayListBuilder& aBuilder, wr::IpcResourceUpdateQueue& aResources,
     const StackingContextHelper& aSc,
-    mozilla::layers::WebRenderLayerManager* aManager,
+    mozilla::layers::RenderRootStateManager* aManager,
     nsDisplayListBuilder* aDisplayListBuilder) {
   // FIXME: avoid needing to make this target if we're drawing text
   // (non-trivial refactor of all this code)
@@ -694,8 +629,14 @@ bool nsDisplayBullet::CreateWebRenderCommands(
     return false;
   }
 
-  return br->CreateWebRenderCommands(this, aBuilder, aResources, aSc, aManager,
-                                     aDisplayListBuilder);
+  ImgDrawResult drawResult = br->CreateWebRenderCommands(
+      this, aBuilder, aResources, aSc, aManager, aDisplayListBuilder);
+  if (drawResult == ImgDrawResult::NOT_SUPPORTED) {
+    return false;
+  }
+
+  nsDisplayBulletGeometry::UpdateDrawResult(this, drawResult);
+  return true;
 }
 
 void nsDisplayBullet::Paint(nsDisplayListBuilder* aBuilder, gfxContext* aCtx) {
@@ -705,40 +646,44 @@ void nsDisplayBullet::Paint(nsDisplayListBuilder* aBuilder, gfxContext* aCtx) {
   }
 
   ImgDrawResult result = static_cast<nsBulletFrame*>(mFrame)->PaintBullet(
-      *aCtx, ToReferenceFrame(), mVisibleRect, flags, mDisableSubpixelAA);
+      *aCtx, ToReferenceFrame(), GetPaintRect(), flags, IsSubpixelAADisabled());
 
   nsDisplayBulletGeometry::UpdateDrawResult(this, result);
 }
 
 void nsBulletFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
                                      const nsDisplayListSet& aLists) {
-  if (!IsVisibleForPainting(aBuilder)) return;
+  if (!IsVisibleForPainting()) return;
 
   DO_GLOBAL_REFLOW_COUNT_DSP("nsBulletFrame");
 
-  aLists.Content()->AppendToTop(
-      MakeDisplayItem<nsDisplayBullet>(aBuilder, this));
+  aLists.Content()->AppendNewToTop<nsDisplayBullet>(aBuilder, this);
 }
 
 Maybe<BulletRenderer> nsBulletFrame::CreateBulletRenderer(
     gfxContext& aRenderingContext, nsPoint aPt) {
   const nsStyleList* myList = StyleList();
-  CounterStyle* listStyleType = myList->mCounterStyle;
+  CounterStyle* listStyleType = ResolveCounterStyle();
   nsMargin padding = mPadding.GetPhysicalMargin(GetWritingMode());
 
   if (myList->GetListStyleImage() && mImageRequest) {
     uint32_t status;
     mImageRequest->GetImageStatus(&status);
-    if (status & imgIRequest::STATUS_LOAD_COMPLETE &&
-        !(status & imgIRequest::STATUS_ERROR)) {
-      nsCOMPtr<imgIContainer> imageCon;
-      mImageRequest->GetImage(getter_AddRefs(imageCon));
-      if (imageCon) {
-        nsRect dest(padding.left, padding.top,
-                    mRect.width - (padding.left + padding.right),
-                    mRect.height - (padding.top + padding.bottom));
-        BulletRenderer br(imageCon, dest + aPt);
-        return Some(br);
+    if (!(status & imgIRequest::STATUS_ERROR)) {
+      if (status & imgIRequest::STATUS_LOAD_COMPLETE) {
+        nsCOMPtr<imgIContainer> imageCon;
+        mImageRequest->GetImage(getter_AddRefs(imageCon));
+        if (imageCon) {
+          nsRect dest(padding.left, padding.top,
+                      mRect.width - (padding.left + padding.right),
+                      mRect.height - (padding.top + padding.bottom));
+          BulletRenderer br(imageCon, dest + aPt);
+          return Some(br);
+        }
+      } else {
+        // Boost the load priority further now that we know we want to display
+        // the bullet image.
+        mImageRequest->BoostPriority(imgIRequest::CATEGORY_DISPLAY);
       }
     }
   }
@@ -839,8 +784,8 @@ Maybe<BulletRenderer> nsBulletFrame::CreateBulletRenderer(
       RefPtr<nsFontMetrics> fm =
           nsLayoutUtils::GetFontMetricsForFrame(this, GetFontSizeInflation());
       nsAutoString text;
-      GetListItemText(text);
       WritingMode wm = GetWritingMode();
+      GetListItemText(listStyleType, wm, Ordinal(), text);
       nscoord ascent = wm.IsLineInverted() ? fm->MaxDescent() : fm->MaxAscent();
       aPt.MoveBy(padding.left, padding.top);
       if (wm.IsVertical()) {
@@ -879,54 +824,27 @@ ImgDrawResult nsBulletFrame::PaintBullet(gfxContext& aRenderingContext,
                    aDisableSubpixelAA, this);
 }
 
-int32_t nsBulletFrame::SetListItemOrdinal(int32_t aNextOrdinal, bool* aChanged,
-                                          int32_t aIncrement) {
-  MOZ_ASSERT(aIncrement == 1 || aIncrement == -1,
-             "We shouldn't have weird increments here");
-
-  // Assume that the ordinal comes from the caller
-  int32_t oldOrdinal = mOrdinal;
-  mOrdinal = aNextOrdinal;
-
-  // Try to get value directly from the list-item, if it specifies a
-  // value attribute. Note: we do this with our parent's content
-  // because our parent is the list-item.
-  nsIContent* parentContent = GetParent()->GetContent();
-  if (parentContent) {
-    nsGenericHTMLElement* hc = nsGenericHTMLElement::FromContent(parentContent);
-    if (hc) {
-      const nsAttrValue* attr = hc->GetParsedAttr(nsGkAtoms::value);
-      if (attr && attr->Type() == nsAttrValue::eInteger) {
-        // Use ordinal specified by the value attribute
-        mOrdinal = attr->GetIntegerValue();
-      }
-    }
-  }
-
-  *aChanged = oldOrdinal != mOrdinal;
-
-  return nsCounterManager::IncrementCounter(mOrdinal, aIncrement);
-}
-
-void nsBulletFrame::GetListItemText(nsAString& aResult) {
-  CounterStyle* style = StyleList()->mCounterStyle;
-  NS_ASSERTION(style->GetStyle() != NS_STYLE_LIST_STYLE_NONE &&
-                   style->GetStyle() != NS_STYLE_LIST_STYLE_DISC &&
-                   style->GetStyle() != NS_STYLE_LIST_STYLE_CIRCLE &&
-                   style->GetStyle() != NS_STYLE_LIST_STYLE_SQUARE &&
-                   style->GetStyle() != NS_STYLE_LIST_STYLE_DISCLOSURE_CLOSED &&
-                   style->GetStyle() != NS_STYLE_LIST_STYLE_DISCLOSURE_OPEN,
-               "we should be using specialized code for these types");
+void nsBulletFrame::GetListItemText(CounterStyle* aStyle,
+                                    mozilla::WritingMode aWritingMode,
+                                    int32_t aOrdinal, nsAString& aResult) {
+  NS_ASSERTION(
+      aStyle->GetStyle() != NS_STYLE_LIST_STYLE_NONE &&
+          aStyle->GetStyle() != NS_STYLE_LIST_STYLE_DISC &&
+          aStyle->GetStyle() != NS_STYLE_LIST_STYLE_CIRCLE &&
+          aStyle->GetStyle() != NS_STYLE_LIST_STYLE_SQUARE &&
+          aStyle->GetStyle() != NS_STYLE_LIST_STYLE_DISCLOSURE_CLOSED &&
+          aStyle->GetStyle() != NS_STYLE_LIST_STYLE_DISCLOSURE_OPEN,
+      "we should be using specialized code for these types");
 
   bool isRTL;
   nsAutoString counter, prefix, suffix;
-  style->GetPrefix(prefix);
-  style->GetSuffix(suffix);
-  style->GetCounterText(mOrdinal, GetWritingMode(), counter, isRTL);
+  aStyle->GetPrefix(prefix);
+  aStyle->GetSuffix(suffix);
+  aStyle->GetCounterText(aOrdinal, aWritingMode, counter, isRTL);
 
   aResult.Truncate();
   aResult.Append(prefix);
-  if (GetWritingMode().IsBidiLTR() != isRTL) {
+  if (aWritingMode.IsBidiLTR() != isRTL) {
     aResult.Append(counter);
   } else {
     // RLM = 0x200f, LRM = 0x200e
@@ -992,7 +910,8 @@ void nsBulletFrame::GetDesiredSize(nsPresContext* aCX,
   nscoord bulletSize;
 
   nsAutoString text;
-  switch (myList->mCounterStyle->GetStyle()) {
+  CounterStyle* style = ResolveCounterStyle();
+  switch (style->GetStyle()) {
     case NS_STYLE_LIST_STYLE_NONE:
       finalSize.ISize(wm) = finalSize.BSize(wm) = 0;
       aMetrics.SetBlockStartAscent(0);
@@ -1025,7 +944,7 @@ void nsBulletFrame::GetDesiredSize(nsPresContext* aCX,
       break;
 
     default:
-      GetListItemText(text);
+      GetListItemText(style, wm, Ordinal(), text);
       finalSize.BSize(wm) = fm->MaxHeight();
       finalSize.ISize(wm) = nsLayoutUtils::AppUnitWidthOfStringBidi(
           text, this, *fm, *aRenderingContext);
@@ -1076,11 +995,11 @@ void nsBulletFrame::Reflow(nsPresContext* aPresContext, ReflowOutput& aMetrics,
   NS_FRAME_SET_TRUNCATION(aStatus, aReflowInput, aMetrics);
 }
 
-/* virtual */ nscoord nsBulletFrame::GetMinISize(
-    gfxContext* aRenderingContext) {
+/* virtual */
+nscoord nsBulletFrame::GetMinISize(gfxContext* aRenderingContext) {
   WritingMode wm = GetWritingMode();
   ReflowOutput reflowOutput(wm);
-  DISPLAY_MIN_WIDTH(this, reflowOutput.ISize(wm));
+  DISPLAY_MIN_INLINE_SIZE(this, reflowOutput.ISize(wm));
   LogicalMargin padding(wm);
   GetDesiredSize(PresContext(), aRenderingContext, reflowOutput, 1.0f,
                  &padding);
@@ -1088,11 +1007,11 @@ void nsBulletFrame::Reflow(nsPresContext* aPresContext, ReflowOutput& aMetrics,
   return reflowOutput.ISize(wm);
 }
 
-/* virtual */ nscoord nsBulletFrame::GetPrefISize(
-    gfxContext* aRenderingContext) {
+/* virtual */
+nscoord nsBulletFrame::GetPrefISize(gfxContext* aRenderingContext) {
   WritingMode wm = GetWritingMode();
   ReflowOutput metrics(wm);
-  DISPLAY_PREF_WIDTH(this, metrics.ISize(wm));
+  DISPLAY_PREF_INLINE_SIZE(this, metrics.ISize(wm));
   LogicalMargin padding(wm);
   GetDesiredSize(PresContext(), aRenderingContext, metrics, 1.0f, &padding);
   metrics.ISize(wm) += padding.IStartEnd(wm);
@@ -1107,11 +1026,12 @@ static inline bool IsIgnoreable(const nsIFrame* aFrame, nscoord aISize) {
     return false;
   }
   auto listStyle = aFrame->StyleList();
-  return listStyle->mCounterStyle->IsNone() && !listStyle->GetListStyleImage();
+  return listStyle->mCounterStyle.IsNone() && !listStyle->GetListStyleImage();
 }
 
-/* virtual */ void nsBulletFrame::AddInlineMinISize(
-    gfxContext* aRenderingContext, nsIFrame::InlineMinISizeData* aData) {
+/* virtual */
+void nsBulletFrame::AddInlineMinISize(gfxContext* aRenderingContext,
+                                      nsIFrame::InlineMinISizeData* aData) {
   nscoord isize = nsLayoutUtils::IntrinsicForContainer(
       aRenderingContext, this, nsLayoutUtils::MIN_ISIZE);
   if (MOZ_LIKELY(!::IsIgnoreable(this, isize))) {
@@ -1119,8 +1039,9 @@ static inline bool IsIgnoreable(const nsIFrame* aFrame, nscoord aISize) {
   }
 }
 
-/* virtual */ void nsBulletFrame::AddInlinePrefISize(
-    gfxContext* aRenderingContext, nsIFrame::InlinePrefISizeData* aData) {
+/* virtual */
+void nsBulletFrame::AddInlinePrefISize(gfxContext* aRenderingContext,
+                                       nsIFrame::InlinePrefISizeData* aData) {
   nscoord isize = nsLayoutUtils::IntrinsicForContainer(
       aRenderingContext, this, nsLayoutUtils::PREF_ISIZE);
   if (MOZ_LIKELY(!::IsIgnoreable(this, isize))) {
@@ -1167,15 +1088,16 @@ nsBulletFrame::Notify(imgIRequest* aRequest, int32_t aType,
       container->GetHeight(&height);
 
       // Request a decode at that size.
-      container->RequestDecodeForSize(IntSize(width, height),
-                                      imgIContainer::DECODE_FLAGS_DEFAULT);
+      container->RequestDecodeForSize(
+          IntSize(width, height), imgIContainer::DECODE_FLAGS_DEFAULT |
+                                      imgIContainer::FLAG_HIGH_QUALITY_SCALING);
     }
 
     InvalidateFrame();
   }
 
   if (aType == imgINotificationObserver::DECODE_COMPLETE) {
-    if (nsIDocument* parent = GetOurCurrentDoc()) {
+    if (Document* parent = GetOurCurrentDoc()) {
       nsCOMPtr<imgIContainer> container;
       aRequest->GetImage(getter_AddRefs(container));
       if (container) {
@@ -1187,7 +1109,7 @@ nsBulletFrame::Notify(imgIRequest* aRequest, int32_t aType,
   return NS_OK;
 }
 
-nsIDocument* nsBulletFrame::GetOurCurrentDoc() const {
+Document* nsBulletFrame::GetOurCurrentDoc() const {
   nsIContent* parentContent = GetParent()->GetContent();
   return parentContent ? parentContent->GetComposedDoc() : nullptr;
 }
@@ -1218,10 +1140,10 @@ nsresult nsBulletFrame::OnSizeAvailable(imgIRequest* aRequest,
 
     // Now that the size is available (or an error occurred), trigger
     // a reflow of the bullet frame.
-    nsIPresShell* shell = presContext->GetPresShell();
-    if (shell) {
-      shell->FrameNeedsReflow(this, nsIPresShell::eStyleChange,
-                              NS_FRAME_IS_DIRTY);
+    mozilla::PresShell* presShell = presContext->GetPresShell();
+    if (presShell) {
+      presShell->FrameNeedsReflow(this, IntrinsicDirty::StyleChange,
+                                  NS_FRAME_IS_DIRTY);
     }
   }
 
@@ -1239,13 +1161,14 @@ void nsBulletFrame::GetLoadGroup(nsPresContext* aPresContext,
                                  nsILoadGroup** aLoadGroup) {
   if (!aPresContext) return;
 
-  NS_PRECONDITION(nullptr != aLoadGroup, "null OUT parameter pointer");
+  MOZ_ASSERT(nullptr != aLoadGroup, "null OUT parameter pointer");
 
-  nsIPresShell* shell = aPresContext->GetPresShell();
+  mozilla::PresShell* presShell = aPresContext->GetPresShell();
+  if (!presShell) {
+    return;
+  }
 
-  if (!shell) return;
-
-  nsIDocument* doc = shell->GetDocument();
+  Document* doc = presShell->GetDocument();
   if (!doc) return;
 
   *aLoadGroup = doc->GetDocumentLoadGroup().take();
@@ -1281,49 +1204,70 @@ already_AddRefed<imgIContainer> nsBulletFrame::GetImage() const {
   return nullptr;
 }
 
+nscoord nsBulletFrame::GetListStyleAscent() const {
+  RefPtr<nsFontMetrics> fm =
+      nsLayoutUtils::GetFontMetricsForFrame(this, GetFontSizeInflation());
+  auto* list = StyleList();
+  if (list->mCounterStyle.IsNone()) {
+    return 0;
+  }
+  if (list->mCounterStyle.IsAnonymous()) {
+    return fm->MaxAscent();
+  }
+  // NOTE(emilio): @counter-style can override most of the styles from this
+  // list, and we still return the changed ascent. Do we care about that?
+  //
+  // https://github.com/w3c/csswg-drafts/issues/3584
+  nsAtom* style = list->mCounterStyle.AsAtom();
+  if (style == nsGkAtoms::disc || style == nsGkAtoms::circle ||
+      style == nsGkAtoms::square) {
+    nscoord ascent = fm->MaxAscent();
+    nscoord baselinePadding = NSToCoordRound(float(ascent) / 8.0f);
+    ascent = std::max(nsPresContext::CSSPixelsToAppUnits(MIN_BULLET_SIZE),
+                      NSToCoordRound(0.8f * (float(ascent) / 2.0f)));
+    return ascent + baselinePadding;
+  }
+  if (style == nsGkAtoms::disclosure_open ||
+      style == nsGkAtoms::disclosure_closed) {
+    nscoord ascent = fm->EmAscent();
+    nscoord baselinePadding = NSToCoordRound(0.125f * ascent);
+    ascent = std::max(nsPresContext::CSSPixelsToAppUnits(MIN_BULLET_SIZE),
+                      NSToCoordRound(0.75f * ascent));
+    return ascent + baselinePadding;
+  }
+  return fm->MaxAscent();
+}
+
 nscoord nsBulletFrame::GetLogicalBaseline(WritingMode aWritingMode) const {
-  nscoord ascent = 0, baselinePadding;
+  nscoord ascent = 0;
   if (GetStateBits() & BULLET_FRAME_IMAGE_LOADING) {
     ascent = BSize(aWritingMode);
   } else {
-    RefPtr<nsFontMetrics> fm =
-        nsLayoutUtils::GetFontMetricsForFrame(this, GetFontSizeInflation());
-    CounterStyle* listStyleType = StyleList()->mCounterStyle;
-    switch (listStyleType->GetStyle()) {
-      case NS_STYLE_LIST_STYLE_NONE:
-        break;
-
-      case NS_STYLE_LIST_STYLE_DISC:
-      case NS_STYLE_LIST_STYLE_CIRCLE:
-      case NS_STYLE_LIST_STYLE_SQUARE:
-        ascent = fm->MaxAscent();
-        baselinePadding = NSToCoordRound(float(ascent) / 8.0f);
-        ascent = std::max(nsPresContext::CSSPixelsToAppUnits(MIN_BULLET_SIZE),
-                          NSToCoordRound(0.8f * (float(ascent) / 2.0f)));
-        ascent += baselinePadding;
-        break;
-
-      case NS_STYLE_LIST_STYLE_DISCLOSURE_CLOSED:
-      case NS_STYLE_LIST_STYLE_DISCLOSURE_OPEN:
-        ascent = fm->EmAscent();
-        baselinePadding = NSToCoordRound(0.125f * ascent);
-        ascent = std::max(nsPresContext::CSSPixelsToAppUnits(MIN_BULLET_SIZE),
-                          NSToCoordRound(0.75f * ascent));
-        ascent += baselinePadding;
-        break;
-
-      default:
-        ascent = fm->MaxAscent();
-        break;
-    }
+    ascent = GetListStyleAscent();
   }
   return ascent + GetLogicalUsedMargin(aWritingMode).BStart(aWritingMode);
 }
 
+bool nsBulletFrame::GetNaturalBaselineBOffset(WritingMode aWM,
+                                              BaselineSharingGroup,
+                                              nscoord* aBaseline) const {
+  nscoord ascent = 0;
+  if (GetStateBits() & BULLET_FRAME_IMAGE_LOADING) {
+    ascent = BSize(aWM);
+  } else {
+    ascent = GetListStyleAscent();
+  }
+  *aBaseline = ascent;
+  return true;
+}
+
+#ifdef ACCESSIBILITY
 void nsBulletFrame::GetSpokenText(nsAString& aText) {
-  CounterStyle* style = StyleList()->mCounterStyle;
+  CounterStyle* style =
+      PresContext()->CounterStyleManager()->ResolveCounterStyle(
+          StyleList()->mCounterStyle);
   bool isBullet;
-  style->GetSpokenCounterText(mOrdinal, GetWritingMode(), aText, isBullet);
+  style->GetSpokenCounterText(Ordinal(), GetWritingMode(), aText, isBullet);
   if (isBullet) {
     if (!style->IsNone()) {
       aText.Append(' ');
@@ -1335,6 +1279,7 @@ void nsBulletFrame::GetSpokenText(nsAString& aText) {
     aText = prefix + aText + suffix;
   }
 }
+#endif
 
 void nsBulletFrame::RegisterImageRequest(bool aKnownToBeAnimated) {
   if (mImageRequest) {
@@ -1350,7 +1295,7 @@ void nsBulletFrame::RegisterImageRequest(bool aKnownToBeAnimated) {
           PresContext(), mImageRequest, &isRequestRegistered);
     }
 
-    isRequestRegistered = mRequestRegistered;
+    mRequestRegistered = isRequestRegistered;
   }
 }
 
@@ -1364,7 +1309,7 @@ void nsBulletFrame::DeregisterAndCancelImageRequest() {
     nsLayoutUtils::DeregisterImageRequest(PresContext(), mImageRequest,
                                           &isRequestRegistered);
 
-    isRequestRegistered = mRequestRegistered;
+    mRequestRegistered = isRequestRegistered;
 
     // Cancel the image request and forget about it.
     mImageRequest->CancelAndForgetObserver(NS_ERROR_FAILURE);
@@ -1372,6 +1317,17 @@ void nsBulletFrame::DeregisterAndCancelImageRequest() {
   }
 }
 
+void nsBulletFrame::SetOrdinal(int32_t aOrdinal, bool aNotify) {
+  if (mOrdinal == aOrdinal) {
+    return;
+  }
+  mOrdinal = aOrdinal;
+  if (aNotify) {
+    PresShell()->FrameNeedsReflow(this, IntrinsicDirty::StyleChange,
+                                  NS_FRAME_IS_DIRTY);
+  }
+}
+
 NS_IMPL_ISUPPORTS(nsBulletListener, imgINotificationObserver)
 
 nsBulletListener::nsBulletListener() : mFrame(nullptr) {}
