@@ -4,17 +4,20 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #include "nsPageContentFrame.h"
+
+#include "mozilla/PresShell.h"
+
 #include "nsCSSFrameConstructor.h"
 #include "nsPresContext.h"
 #include "nsGkAtoms.h"
-#include "nsIPresShell.h"
 #include "nsSimplePageSequenceFrame.h"
 
 using namespace mozilla;
 
-nsPageContentFrame* NS_NewPageContentFrame(nsIPresShell* aPresShell,
-                                           nsStyleContext* aContext) {
-  return new (aPresShell) nsPageContentFrame(aContext);
+nsPageContentFrame* NS_NewPageContentFrame(PresShell* aPresShell,
+                                           ComputedStyle* aStyle) {
+  return new (aPresShell)
+      nsPageContentFrame(aStyle, aPresShell->GetPresContext());
 }
 
 NS_IMPL_FRAMEARENA_HELPERS(nsPageContentFrame)
