@@ -4,9 +4,13 @@
 
 "use strict";
 
-var {DelayedInit} = ChromeUtils.import("resource://gre/modules/DelayedInit.jsm");
-var {XPCOMUtils} = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var { DelayedInit } = ChromeUtils.import(
+  "resource://gre/modules/DelayedInit.jsm"
+);
+var { XPCOMUtils } = ChromeUtils.import(
+  "resource://gre/modules/XPCOMUtils.jsm"
+);
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 XPCOMUtils.defineLazyModuleGetters(this, {
   E10SUtils: "resource://gre/modules/E10SUtils.jsm",
@@ -15,8 +19,9 @@ XPCOMUtils.defineLazyModuleGetters(this, {
   HistogramStopwatch: "resource://gre/modules/GeckoViewTelemetry.jsm",
 });
 
-XPCOMUtils.defineLazyGetter(this, "WindowEventDispatcher",
-  () => EventDispatcher.for(window));
+XPCOMUtils.defineLazyGetter(this, "WindowEventDispatcher", () =>
+  EventDispatcher.for(window)
+);
 
 /**
  * ModuleManager creates and manages GeckoView modules. Each GeckoView module
@@ -33,8 +38,10 @@ var ModuleManager = {
   },
 
   init(aBrowser, aModules) {
-    const MODULES_INIT_PROBE =
-      new HistogramStopwatch("GV_STARTUP_MODULES_MS", aBrowser);
+    const MODULES_INIT_PROBE = new HistogramStopwatch(
+      "GV_STARTUP_MODULES_MS",
+      aBrowser
+    );
 
     MODULES_INIT_PROBE.start();
 
@@ -44,18 +51,20 @@ var ModuleManager = {
     this._frozenSettings = Object.freeze(Object.assign({}, this._settings));
 
     const self = this;
-    this._modules = new Map((function* () {
-      for (const module of aModules) {
-        yield [
-          module.name,
-          new ModuleInfo({
-            enabled: !!initData.modules[module.name],
-            manager: self,
-            ...module,
-          }),
-        ];
-      }
-    })());
+    this._modules = new Map(
+      (function*() {
+        for (const module of aModules) {
+          yield [
+            module.name,
+            new ModuleInfo({
+              enabled: !!initData.modules[module.name],
+              manager: self,
+              ...module,
+            }),
+          ];
+        }
+      })()
+    );
 
     window.document.documentElement.appendChild(aBrowser);
 
@@ -65,8 +74,10 @@ var ModuleManager = {
       "GeckoView:UpdateSettings",
     ]);
 
-    this.messageManager.addMessageListener("GeckoView:ContentModuleLoaded",
-                                           this);
+    this.messageManager.addMessageListener(
+      "GeckoView:ContentModuleLoaded",
+      this
+    );
 
     this.forEach(module => {
       module.onInit();
@@ -110,14 +121,16 @@ var ModuleManager = {
   },
 
   updateRemoteTypeForURI(aURI) {
-    const currentType =
-        this.browser.remoteType || E10SUtils.NOT_REMOTE;
+    const currentType = this.browser.remoteType || E10SUtils.NOT_REMOTE;
     const remoteType = E10SUtils.getRemoteTypeForURI(
-        aURI, this.settings.useMultiprocess,
-        /* useRemoteSubframes */ false,
-        currentType, this.browser.currentURI);
+      aURI,
+      this.settings.useMultiprocess,
+      /* useRemoteSubframes */ false,
+      currentType,
+      this.browser.currentURI
+    );
 
-    debug `updateRemoteType: uri=${aURI} currentType=${currentType}
+    debug`updateRemoteType: uri=${aURI} currentType=${currentType}
                              remoteType=${remoteType}`;
 
     if (currentType === remoteType) {
@@ -125,9 +138,8 @@ var ModuleManager = {
       return false;
     }
 
-    if (remoteType !== E10SUtils.NOT_REMOTE &&
-        !this.settings.useMultiprocess) {
-      warn `Tried to create a remote browser in non-multiprocess mode`;
+    if (remoteType !== E10SUtils.NOT_REMOTE && !this.settings.useMultiprocess) {
+      warn`Tried to create a remote browser in non-multiprocess mode`;
       return false;
     }
 
@@ -163,8 +175,10 @@ var ModuleManager = {
 
     parent.appendChild(this.browser);
 
-    this.messageManager.addMessageListener("GeckoView:ContentModuleLoaded",
-                                           this);
+    this.messageManager.addMessageListener(
+      "GeckoView:ContentModuleLoaded",
+      this
+    );
 
     this.forEach(module => {
       // We're attaching a new browser so we have to reload the frame scripts
@@ -189,12 +203,14 @@ var ModuleManager = {
       }
     });
 
-    this._browser.messageManager.sendAsyncMessage("GeckoView:UpdateSettings",
-                                                  aSettings);
+    this._browser.messageManager.sendAsyncMessage(
+      "GeckoView:UpdateSettings",
+      aSettings
+    );
   },
 
   onEvent(aEvent, aData, aCallback) {
-    debug `onEvent ${aEvent} ${aData}`;
+    debug`onEvent ${aEvent} ${aData}`;
     switch (aEvent) {
       case "GeckoView:UpdateModuleState": {
         const module = this._modules.get(aData.module);
@@ -230,7 +246,7 @@ var ModuleManager = {
   },
 
   receiveMessage(aMsg) {
-    debug `receiveMessage ${aMsg.name} ${aMsg.data}`;
+    debug`receiveMessage ${aMsg.name} ${aMsg.data}`;
     switch (aMsg.name) {
       case "GeckoView:ContentModuleLoaded": {
         const module = this._modules.get(aMsg.data.module);
@@ -260,7 +276,7 @@ class ModuleInfo {
    * @param onEnable Phase object for the enable phase, when the module is first
    *                 enabled by setting a delegate in Java.
    */
-  constructor({manager, name, enabled, onInit, onEnable}) {
+  constructor({ manager, name, enabled, onInit, onEnable }) {
     this._manager = manager;
     this._name = name;
 
@@ -391,16 +407,19 @@ class ModuleInfo {
   }
 
   _updateContentModuleState(aIncludeSettings) {
-    this._manager.messageManager.sendAsyncMessage("GeckoView:UpdateModuleState", {
-      module: this._name,
-      enabled: this.enabled,
-      settings: aIncludeSettings ? this._manager.settings : null,
-    });
+    this._manager.messageManager.sendAsyncMessage(
+      "GeckoView:UpdateModuleState",
+      {
+        module: this._name,
+        enabled: this.enabled,
+        settings: aIncludeSettings ? this._manager.settings : null,
+      }
+    );
   }
 }
 
 function createBrowser() {
-  const browser = window.browser = document.createElement("browser");
+  const browser = (window.browser = document.createElement("browser"));
   // Identify this `<browser>` element uniquely to Marionette, devtools, etc.
   browser.permanentKey = {};
 
@@ -408,7 +427,8 @@ function createBrowser() {
   browser.setAttribute("primary", "true");
   browser.setAttribute("flex", "1");
 
-  const settings = window.arguments[0].QueryInterface(Ci.nsIAndroidView).initData.settings;
+  const settings = window.arguments[0].QueryInterface(Ci.nsIAndroidView)
+    .initData.settings;
   if (settings.useMultiprocess) {
     browser.setAttribute("remote", "true");
     browser.setAttribute("remoteType", E10SUtils.DEFAULT_REMOTE_TYPE);
@@ -425,67 +445,80 @@ function startup() {
   GeckoViewUtils.initLogging("XUL", window);
 
   const browser = createBrowser();
-  ModuleManager.init(browser, [{
-    name: "GeckoViewAccessibility",
-    onInit: {
-      resource: "resource://gre/modules/GeckoViewAccessibility.jsm",
+  ModuleManager.init(browser, [
+    {
+      name: "GeckoViewAccessibility",
+      onInit: {
+        resource: "resource://gre/modules/GeckoViewAccessibility.jsm",
+      },
     },
-  }, {
-    name: "GeckoViewContent",
-    onInit: {
-      resource: "resource://gre/modules/GeckoViewContent.jsm",
-      frameScript: "chrome://geckoview/content/GeckoViewContentChild.js",
+    {
+      name: "GeckoViewContent",
+      onInit: {
+        resource: "resource://gre/modules/GeckoViewContent.jsm",
+        frameScript: "chrome://geckoview/content/GeckoViewContentChild.js",
+      },
     },
-  }, {
-    name: "GeckoViewMedia",
-    onEnable: {
-      resource: "resource://gre/modules/GeckoViewMedia.jsm",
-      frameScript: "chrome://geckoview/content/GeckoViewMediaChild.js",
+    {
+      name: "GeckoViewMedia",
+      onEnable: {
+        resource: "resource://gre/modules/GeckoViewMedia.jsm",
+        frameScript: "chrome://geckoview/content/GeckoViewMediaChild.js",
+      },
     },
-  }, {
-    name: "GeckoViewNavigation",
-    onInit: {
-      resource: "resource://gre/modules/GeckoViewNavigation.jsm",
-      frameScript: "chrome://geckoview/content/GeckoViewNavigationChild.js",
+    {
+      name: "GeckoViewNavigation",
+      onInit: {
+        resource: "resource://gre/modules/GeckoViewNavigation.jsm",
+        frameScript: "chrome://geckoview/content/GeckoViewNavigationChild.js",
+      },
     },
-  }, {
-    name: "GeckoViewProgress",
-    onEnable: {
-      resource: "resource://gre/modules/GeckoViewProgress.jsm",
-      frameScript: "chrome://geckoview/content/GeckoViewProgressChild.js",
+    {
+      name: "GeckoViewProgress",
+      onEnable: {
+        resource: "resource://gre/modules/GeckoViewProgress.jsm",
+        frameScript: "chrome://geckoview/content/GeckoViewProgressChild.js",
+      },
     },
-  }, {
-    name: "GeckoViewScroll",
-    onEnable: {
-      frameScript: "chrome://geckoview/content/GeckoViewScrollChild.js",
+    {
+      name: "GeckoViewScroll",
+      onEnable: {
+        frameScript: "chrome://geckoview/content/GeckoViewScrollChild.js",
+      },
     },
-  }, {
-    name: "GeckoViewSelectionAction",
-    onEnable: {
-      frameScript: "chrome://geckoview/content/GeckoViewSelectionActionChild.js",
+    {
+      name: "GeckoViewSelectionAction",
+      onEnable: {
+        frameScript:
+          "chrome://geckoview/content/GeckoViewSelectionActionChild.js",
+      },
     },
-  }, {
-    name: "GeckoViewSettings",
-    onInit: {
-      resource: "resource://gre/modules/GeckoViewSettings.jsm",
-      frameScript: "chrome://geckoview/content/GeckoViewSettingsChild.js",
+    {
+      name: "GeckoViewSettings",
+      onInit: {
+        resource: "resource://gre/modules/GeckoViewSettings.jsm",
+        frameScript: "chrome://geckoview/content/GeckoViewSettingsChild.js",
+      },
     },
-  }, {
-    name: "GeckoViewTab",
-    onInit: {
-      resource: "resource://gre/modules/GeckoViewTab.jsm",
+    {
+      name: "GeckoViewTab",
+      onInit: {
+        resource: "resource://gre/modules/GeckoViewTab.jsm",
+      },
     },
-  }, {
-    name: "GeckoViewContentBlocking",
-    onEnable: {
-      resource: "resource://gre/modules/GeckoViewContentBlocking.jsm",
+    {
+      name: "GeckoViewContentBlocking",
+      onEnable: {
+        resource: "resource://gre/modules/GeckoViewContentBlocking.jsm",
+      },
     },
-  }, {
-    name: "SessionStateAggregator",
-    onInit: {
-      frameScript: "chrome://geckoview/content/SessionStateAggregator.js",
+    {
+      name: "SessionStateAggregator",
+      onInit: {
+        frameScript: "chrome://geckoview/content/SessionStateAggregator.js",
+      },
     },
-  }]);
+  ]);
 
   Services.tm.dispatchToMainThread(() => {
     // This should always be the first thing we do here - any additional delayed
@@ -496,7 +529,9 @@ function startup() {
     // to learn when a browser window has finished its initial (chrome)
     // initialisation, especially with regards to the very first window that is
     // created. Therefore, GeckoView "windows" need to send this, too.
-    InitLater(() => Services.obs.notifyObservers(window, "browser-delayed-startup-finished"));
+    InitLater(() =>
+      Services.obs.notifyObservers(window, "browser-delayed-startup-finished")
+    );
 
     // This should always go last, since the idle tasks (except for the ones with
     // timeouts) should execute in order. Note that this observer notification is
@@ -505,7 +540,12 @@ function startup() {
     // This notification in particular signals the ScriptPreloader that we have
     // finished startup, so it can now stop recording script usage and start
     // updating the startup cache for faster script loading.
-    InitLater(() => Services.obs.notifyObservers(window, "browser-idle-startup-tasks-finished"));
+    InitLater(() =>
+      Services.obs.notifyObservers(
+        window,
+        "browser-idle-startup-tasks-finished"
+      )
+    );
   });
 
   // Move focus to the content window at the end of startup,
