@@ -142,13 +142,19 @@ public class SessionAccessibility {
 
             switch (action) {
                 case AccessibilityNodeInfo.ACTION_CLEAR_ACCESSIBILITY_FOCUS:
+                    if (mAccessibilityFocusedNode == virtualViewId) {
+                        mSession.getEventDispatcher().dispatch("GeckoView:AccessibilityClearCursor", null);
+                    }
                     sendEvent(AccessibilityEvent.TYPE_VIEW_ACCESSIBILITY_FOCUS_CLEARED, virtualViewId, CLASSNAME_UNKNOWN, null);
                     return true;
                 case AccessibilityNodeInfo.ACTION_ACCESSIBILITY_FOCUS:
                     if (virtualViewId == View.NO_ID) {
                         sendEvent(AccessibilityEvent.TYPE_VIEW_ACCESSIBILITY_FOCUSED, View.NO_ID, CLASSNAME_WEBVIEW, null);
                     } else {
-                        if (mFocusedNode == virtualViewId) {
+                        if (mFocusedNode == virtualViewId && mHoveredOnNode != virtualViewId) {
+                            // If we are sending accessibility focus to the focused node, sync up the state with Gecko.
+                            // XXX: This is a stopgap for now until we remove the JS layer and manipulate the Gecko a11y virtual cursor directly
+                            // with the given virtualViewId
                             mSession.getEventDispatcher().dispatch("GeckoView:AccessibilityCursorToFocused", null);
                         } else {
                             sendEvent(AccessibilityEvent.TYPE_VIEW_ACCESSIBILITY_FOCUSED, virtualViewId, CLASSNAME_UNKNOWN, null);
@@ -500,6 +506,8 @@ public class SessionAccessibility {
     private int mAccessibilityFocusedNode = 0;
     // The current node with focus
     private int mFocusedNode = 0;
+    // A node with no accessibility focus that is currently being hovered.
+    private int mHoveredOnNode = 0;
     // Viewport cache
     final SparseArray<GeckoBundle> mViewportCache = new SparseArray<>();
     // Focus cache
@@ -760,8 +768,12 @@ public class SessionAccessibility {
                     mAccessibilityFocusedNode = 0;
                 }
                 break;
+            case AccessibilityEvent.TYPE_VIEW_HOVER_ENTER:
+                mHoveredOnNode = sourceId;
+                break;
             case AccessibilityEvent.TYPE_VIEW_ACCESSIBILITY_FOCUSED:
                 mAccessibilityFocusedNode = sourceId;
+                mHoveredOnNode = 0;
                 break;
             case AccessibilityEvent.TYPE_VIEW_FOCUSED:
                 mFocusedNode = sourceId;
